@@ -35,57 +35,62 @@ def parse_nested_q(field_name: str, nested_q: BaseModel):
     return f'{field_name}: {{ {",".join(outer_lst)} }}'
 
 
-Model = T.TypeVar("Model", bound=Node)
-FilterType = T.TypeVar("FilterType", bound=DGraphModel)
-OrderType = T.TypeVar("OrderType", bound=DGraphModel)
+class Params(BaseModel):
+    def to_str(self) -> str:
+        field_names = self.dict(exclude_none=True).keys()
+        inner_params: T.List[str] = []
+        for field_name in field_names:
+            val = getattr(self, field_name)
+            if isinstance(val, BaseModel):
+                inner_params.append(parse_nested_q(field_name=field_name, nested_q=val))
+            else:
+                inner_params.append(
+                    f"{field_name}: {json.dumps(jsonable_encoder(val))}"
+                )
+        if inner_params:
+            return f'({",".join(inner_params)})'
+        return ""
 
 
-class Resolver(BaseModel, T.Generic[Model, FilterType, OrderType]):
-    class Input(BaseModel):
-        def to_str(self) -> str:
-            field_names = self.dict(exclude_none=True).keys()
-            inner_params: T.List[str] = []
-            for field_name in field_names:
-                val = getattr(self, field_name)
-                if isinstance(val, BaseModel):
-                    inner_params.append(
-                        parse_nested_q(field_name=field_name, nested_q=val)
-                    )
-                else:
-                    inner_params.append(
-                        f"{field_name}: {json.dumps(jsonable_encoder(val))}"
-                    )
-            if inner_params:
-                return f'({",".join(inner_params)})'
-            return ""
+NodeType = T.TypeVar("NodeType", bound=Node)
+GetParamsType = T.TypeVar("GetParamsType", bound=Params)
+QueryParamsType = T.TypeVar("QueryParamsType", bound=Params)
+EdgesType = T.TypeVar("EdgesType", bound=BaseModel)
 
-    class Children(BaseModel):
-        pass
 
-    class QueryInput(Input):
-        filter: FilterType = None
-        first: int = None
-        offset: int = None
-        order: OrderType = None
+class Resolver(
+    BaseModel, T.Generic[NodeType, GetParamsType, QueryParamsType, EdgesType]
+):
+    def __init__(
+        self,
+        query_params: T.Optional[QueryParamsType] = None,
+        edges: T.Optional[EdgesType] = None,
+    ):
+        get_params = self.model.GQL.GetParams()
+        query_params = query_params or self.model.GQL.QueryParams()
+        edges = edges or self.model.GQL.Edges()
+        super().__init__(get_params=get_params, query_params=query_params, edges=edges)
 
-    model: T.ClassVar[T.Type[Model]]
-    children_resolvers: Children = Field(default_factory=Children)
-    query_input: QueryInput = Field(default_factory=QueryInput)
+    model: T.ClassVar[T.Type[NodeType]]
+
+    edges: EdgesType = Field(default_factory=Node.GQL.Edges)
+    get_params: GetParamsType = Field(default_factory=Node.GQL.GetParams)
+    query_params: QueryParamsType = Field(default_factory=Node.GQL.QueryParams)
 
     def gql_fields_str(self) -> str:
         """This does not include the top level..."""
         fields = [*self.model.__fields__.keys(), "__typename"]
-        for resolver_name in self.Children.__fields__.keys():
-            resolver: Resolver = getattr(self.children_resolvers, resolver_name, None)
+        for resolver_name in self.edges.__fields__.keys():
+            resolver: T.Optional[Resolver] = getattr(self.edges, resolver_name, None)
             if resolver:
                 child_gql_str = resolver.params_and_fields()
                 fields.append(f"{resolver_name} {child_gql_str}")
         return f'{{ {",".join(fields)} }}'
 
     def params_and_fields(self) -> str:
-        return f"{self.query_input.to_str()}{self.gql_fields_str()}"
+        return f"{self.query_params.to_str()}{self.gql_fields_str()}"
 
-    def _get(self, kwargs_d: dict) -> T.Optional[Model]:
+    def _get(self, kwargs_d: dict) -> T.Optional[NodeType]:
         kwargs = {k: v for k, v in kwargs_d.items() if v is not None}
         if not kwargs:
             raise GQLException(
@@ -105,65 +110,50 @@ class Resolver(BaseModel, T.Generic[Model, FilterType, OrderType]):
             return self.parse_obj_nested(obj)
         return None
 
-    def query(self) -> T.List[Model]:
+    def query(self) -> T.List[NodeType]:
         query_name = f"query{self.model.__name__}"
         s = f"{{ {query_name}{self.params_and_fields()} }}"
         print(s)
         lst: T.List[dict] = execute(query_str=s)["data"][query_name]
         return [self.parse_obj_nested(d) for d in lst]
 
-    def parse_obj_nested(self, gql_d: dict) -> T.Optional[Model]:
-        """Each obj will be its own. If you want to change this, consider changing the parse"""
-        if not gql_d:
-            return None
-        typename = gql_d.get("__typename", None)
-        if not typename:
-            raise GQLException(f"No typename in {gql_d=}")
-        nodes_by_typename = Node.nodes_by_typename()  # TODO precompute?
-        o: Node = nodes_by_typename[typename].parse_obj(gql_d)
-        other_fields = set(gql_d.keys()) - set(o.__fields__.keys()) - {"__typename"}
+    def parse_obj_nested(self, gql_d: dict) -> NodeType:
+        node: NodeType = self.model.parse_obj(gql_d)
+        other_fields = set(gql_d.keys()) - set(node.__fields__.keys()) - {"__typename"}
         for field in other_fields:
-            resolver = getattr(self.children_resolvers, field, None)
+            resolver = getattr(self.edges, field, None)
             if not resolver:
                 raise GQLException(f"No resolver {field} found!")
             nested_d = gql_d[field]
-            val = (
-                [resolver.parse_obj_nested(d) for d in nested_d]
-                if type(nested_d) == list
-                else resolver.parse_obj_nested(nested_d)
-            )
-            o.cache.add(key=field, resolver=resolver, val=val, gql_d=gql_d)
-        return o
+            value_to_save = nested_d
+            if nested_d:
+                val = (
+                    [resolver.parse_obj_nested(d) for d in nested_d]
+                    if type(nested_d) == list
+                    else resolver.parse_obj_nested(nested_d)
+                )
+                value_to_save = val
+            node.cache.add(key=field, resolver=resolver, val=value_to_save, gql_d=gql_d)
+        return node
+
+    def get(self, get_params: GetParamsType) -> T.Optional[NodeType]:
+        return self._get(kwargs_d=get_params.dict())
+
+    def gerror(self, get_params: GetParamsType) -> NodeType:
+        node = self.get(get_params=get_params)
+        if not node:
+            raise GQLException(f"Node with {get_params=} was None")
+        return node
 
     @staticmethod
     def resolvers_by_typename() -> T.Dict[str, T.Type[Resolver]]:
         d = {}
         subs = Resolver.__subclasses__()
         for sub in subs:
-            typename = sub.model._typename
+            typename = sub.model.GQL.typename
             if typename in d:
                 raise GQLException(
                     f"Two Resolvers share the typename {typename}: ({sub.__name__}, {d[typename].__name__})"
                 )
             d[typename] = sub
         return d
-
-    def first(self, _: int) -> Resolver:
-        self.query_input.first = _
-        return self
-
-    def offset(self, _: int) -> Resolver:
-        self.query_input.offset = _
-        return self
-
-    def filter(self, _: FilterType) -> Resolver:
-        self.query_input.filter = _
-        return self
-
-    def order(self, _: OrderType) -> Resolver:
-        self.query_input.order = _
-        return self
-
-    def get(self, id: str = None) -> Model:
-        kwargs_d = {"id": id}
-        return self._get(kwargs_d=kwargs_d)
